//
// GImage: image handling using gdk-pixbuf
//
//    Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

#include "GImage.h"

#include <stdio.h>

GImage::GImage()
{
    mImage = 0;
}

GImage::GImage(const char* filename)
{
    mImage = 0;
    LoadFromFile(filename);
}

GImage::~GImage()
{
    if (mImage)
        gdk_pixbuf_unref(mImage);
}

int GImage::LoadFromFile(const char* filename)
{
    if (mImage)
        gdk_pixbuf_unref(mImage);
    mImage = gdk_pixbuf_new_from_file(filename);
    if (!mImage)
    {
        fprintf(stderr, "Can't open %s\n", filename);
        return -1;
    }

    mX = gdk_pixbuf_get_width(mImage);
    mY = gdk_pixbuf_get_height(mImage);

    return 0;
}

/* ShowImage assumes gImage already contains the right GdkPixbuf.
 */
void GImage::Draw(GtkWidget* drawingArea)
{
    //printf("GImage::Draw\n");
    if (!mImage)
        return;

    if (drawingArea->window == 0)
        return;

    //gdk_window_resize(win->window, mX, mY);

    gdk_pixbuf_render_to_drawable(mImage, drawingArea->window,
                     drawingArea->style->fg_gc[GTK_WIDGET_STATE(drawingArea)],
                                  0, 0, 0, 0, mX, mY,
                                  GDK_RGB_DITHER_NONE, 0, 0);
}

