//
// GMoonCanvas.cpp
// Draw the moon and related info, using gdk_pixbuf.
//
//    Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

#include "GMoonCanvas.h"

#include "GImage.h"
#include "MoonFeature.h"

//#include <gdk/gdktypes.h>

#include <string.h>
#include <stdio.h>

GMoonCanvas::GMoonCanvas( GtkWidget* canvas, GtkWidget* win )
{
    mCanvas = canvas;
    mWin = win;
    mMoonImg = 0;
}

GMoonCanvas::~GMoonCanvas()
{
    if (mMoonImg)
        delete mMoonImg;
}

void GMoonCanvas::ToggleMirrored()
{
    if (mMirrored < 0)
        mMirrored = 1;
    else
        mMirrored = -1;
    if (mMoonImg)
        delete mMoonImg;
    mMoonImg = 0;
}

void GMoonCanvas::Zoom(int level)
{
    if (level != 0)
        printf("Sorry, can't zoom yet.\n");
        // Don't necessarily return -- we might also be mirroring
    delete mMoonImg;
    mMoonImg = new GImage( (mMirrored >= 0)
                           ? DATADIR "fullmoon.png"
                           : DATADIR "fullmoon-mirror.png" );

    mMoonCenterX = mMoonImg->Width()/2;
    mMoonCenterY = mMoonImg->Height()/2;
    mMoonRadius = mMoonCenterX - 2;

    mImgLibrationE = 4;
    mImgLibrationN = 1;
}

void GMoonCanvas::Clear()
{
    gdk_window_clear(mCanvas->window);
}

void GMoonCanvas::Paint()
{
    if (mMoonImg && (mMoonImg->Width() <= 0 || mMoonImg->Height() <= 0))
    {
        delete mMoonImg;
        mMoonImg = 0;
    }
    if (!mMoonImg)
        return;

    Clear();
    mMoonImg->Draw(mCanvas);

    PaintFeatures();
}

void GMoonCanvas::SetColor(int r, int g, int b)
{
    //printf("SetColor %d %d %d\n", r, g, b);
    GdkColor gcolor;
    //gcolor.pixel = 0;
    gcolor.red = r;
    gcolor.green = g;
    gcolor.blue = b;
    gdk_colormap_alloc_color(gdk_colormap_get_system(), &gcolor, FALSE, TRUE);
    //printf("Color is now %ld, %d, %d, %d\n", gcolor.pixel,
    //       gcolor.red, gcolor.green, gcolor.blue);
    // Since color allocation doesn't work, use white for now:
    gdk_color_white(gdk_colormap_get_system(), &gcolor);
    gdk_gc_set_foreground(mCanvas->style->fg_gc[GTK_STATE_NORMAL], &gcolor);
}

void GMoonCanvas::DrawLine(int x1, int y1, int x2, int y2)
{
    gdk_draw_line(mCanvas->window,
                  mCanvas->style->fg_gc[GTK_WIDGET_STATE(mCanvas)],
                  x1, y1, x2, y2);
}

void GMoonCanvas::DrawEllipse(int x, int y, int w, int h)
{
    gdk_draw_arc(mCanvas->window,
                 mCanvas->style->fg_gc[GTK_WIDGET_STATE(mCanvas)],
                 0, x, y, w, h, 0, 23040);
    // angle is specified in 1/64ths of a degree
}

void GMoonCanvas::DrawString(char* str, int x, int y)
{
    gdk_draw_text(mCanvas->window, mCanvas->style->font,
                  mCanvas->style->fg_gc[GTK_WIDGET_STATE(mCanvas)],
                  x, y, str, strlen(str));
}

int GMoonCanvas::Width()
{
    if (mMoonImg)
        return mMoonImg->Width();
    return 100;
}

int GMoonCanvas::Height()
{
    if (mMoonImg)
        return mMoonImg->Height();
    return 100;
}

// The event handler callbacks:
void GMoonCanvas::HandleExpose(GtkWidget* widget, GdkEventExpose* event,
                               gpointer data)
{
    if (!data)
        return;
    ((GMoonCanvas *)data)->Paint();
}

// Static event handlers:
void GMoonCanvas::HandleMouseDown(GtkWidget* widget, GdkEventButton* event,
                                  gpointer data)
{
    if (!data)
        return;
    int numclicks;
    switch (event->type)
    {
      case GDK_2BUTTON_PRESS:
          numclicks = 2;
          break;
      case GDK_3BUTTON_PRESS:
          numclicks = 3;
          break;
      case GDK_BUTTON_PRESS:
      default:
          numclicks = 1;
          break;
    }
    ((GMoonCanvas *)data)->MousePress((int)(event->x), (int)(event->y),
                                      event->button, numclicks);
}

void GMoonCanvas::HandleMouseUp(GtkWidget* widget, GdkEventButton* event,
                                gpointer data)
{
    if (!data)
        return;
    ((GMoonCanvas *)data)->MouseRelease((int)(event->x), (int)(event->y),
                                        event->button, 1);
    // There's no GDK_2BUTTON_RELEASE (or 3) so just return 1 for now.
}



