/*
 * gmain.c: gtk main routines for gMoon
 *
 * Copyright 2002 by Akkana Peck.
 * You are free to use or modify this code under the Gnu Public License.
 */

#include <stdlib.h>       // for getenv()
#include <stdio.h>

#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "GMoonCanvas.h"

static GtkWidget *win = 0;
static GtkWidget *drawingArea = 0;

static GMoonCanvas *moonCanvas = 0;
static int Debug = 0;

static gint HandleDestroy(GtkWidget* widget,
                          GdkEventKey* event, gpointer data)
{
    gtk_main_quit();
    return TRUE;
}

static gint HandleKeyPress(GtkWidget* widget,
                           GdkEventKey* event)
{
    switch (event->keyval)
    {
      case GDK_Escape:
      case GDK_q:
          gtk_main_quit();
          return TRUE;
      default:
          if (Debug)
              printf("Don't know key 0x%lu\n",
                     (unsigned long)(event->keyval));
          return FALSE;
    }
    // Keep gcc 2.95 happy:
    return FALSE;
}

int main(int argc, char** argv)
{
    while (argc > 1 && argv[1][0] == '-')
    {
        if (argv[1][1] == 'd')
            Debug = 1;
        --argc;
        ++argv;
    }

    // See http://www.gtk.org/tutorial
    gtk_init(&argc, &argv);
    
    win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    
    /* Window manager delete */
    //gtk_signal_connect(GTK_OBJECT(win), "delete_event",
    //                   (GtkSignalFunc)HandleDestroy, 0);

    /* This event occurs when we call gtk_widget_destroy() on the window,
     * or if we return FALSE in the "delete_event" callback.
     */
    gtk_signal_connect(GTK_OBJECT(win), "destroy",
                       GTK_SIGNAL_FUNC(HandleDestroy), 0);

    drawingArea = gtk_drawing_area_new();

    moonCanvas = new GMoonCanvas(drawingArea, win);
    moonCanvas->SetDebugMode(Debug);

    /* KeyPress events on the drawing area don't come through --
     * they have to be on the window.
     */
    gtk_widget_set_events(win,
                          GDK_EXPOSURE_MASK | GDK_KEY_PRESS_MASK
                          | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
                          | GDK_STRUCTURE_MASK | GDK_PROPERTY_CHANGE_MASK );
    gtk_signal_connect(GTK_OBJECT(win), "key_press_event",
                       GTK_SIGNAL_FUNC(HandleKeyPress), 0);
    gtk_signal_connect(GTK_OBJECT(win), "button_press_event",
                       GTK_SIGNAL_FUNC(GMoonCanvas::HandleMouseDown),
                       moonCanvas);
    gtk_signal_connect(GTK_OBJECT(win), "button_release_event",
                       GTK_SIGNAL_FUNC(GMoonCanvas::HandleMouseUp),
                       moonCanvas);

    gtk_container_add(GTK_CONTAINER(win), drawingArea);
    gtk_widget_show(drawingArea);

    gtk_signal_connect(GTK_OBJECT(drawingArea), "expose_event",
                       GTK_SIGNAL_FUNC(GMoonCanvas::HandleExpose), moonCanvas);

    //MonitorWidth = gdk_screen_width();
    //MonitorHeight = gdk_screen_height();

    // Must init rgb system explicitly, else we'll crash
    // in the first gdk_pixbuf_render_to_drawable(),
    // calling gdk_draw_rgb_image_dithalign():
    gdk_rgb_init();

    moonCanvas->Init();

    gtk_window_set_title(GTK_WINDOW(win), "The Moon");
    gtk_window_set_default_size(GTK_WINDOW(win),
                                moonCanvas->Width(), moonCanvas->Height());

    // Set the window size before first expose:
    //gdk_window_resize(win->window, moonCanvas->Width(), moonCanvas->Height());

    // Now we know we have something to show, so do it:
    gtk_widget_show(win);

    gtk_main();
    return 0;
}

