//
// MoonFeature.h
// A feature (crater, mare, rille, etc.) on the moon.
//
//    Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

#ifndef MOONFEATURE_H
#define MOONFEATURE_H

class MoonCanvas;

class MoonFeature
{
public:
    MoonFeature();
    virtual ~MoonFeature();

    virtual char* GetName();
    virtual char* GetDesc();
    virtual int GetLongitude();
    virtual int GetLatitude();
    virtual int GetSize();
    virtual int GetSizeX();
    virtual int GetSizeY();

    // Paint the feature and a label.
    // This also chooses a color, which may differ by type.
    virtual void PaintFeature(MoonCanvas* canvas, int cX, int cY, int radius,
                              int mirrored, int librationE, int librationN);
    virtual void LabelName(MoonCanvas* canvas, int x, int y);

    // Find the distance from a point to the object's center.
    // If the point is inside the crater, we also set isInside.
    virtual double Distance(double longitude, double latitude, int* isInside);

    // Routines for looping over all known craters:
    static void InitMoonFeatures(int moonRadius=100);
    static void InitLoop();
    static MoonFeature* NextFeature();

    static void SetCanvasSize(int width, int height);

protected:
    MoonFeature(char* name, char* description, int size,
                int longitude, int latitude);

    void Init(char* name, char* description, int size,
              int longitude, int latitude);

    // We own all the strings used here, and must free them at exit.
    char* mName;
    char* mDesc;

    int mSize;
    int mLongitude;
    int mLatitude;

    // Statics, used for initializing the known list of features.
    static void DeleteFeatures();
    static int sMoonRadius;
    static int sCurIndex;
    static MoonFeature** sMoonFeatureList;
    static int sNumFeatures;

    // And the size of our MoonCanvas, so we can adjust labels:
    static int sWidth, sHeight;
};

#endif // MOONFEATURE_H
