//
// Mountain.cpp
// A mountain range on the moon.
//
//    Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

#include "Mountain.h"

#include <MoonCanvas.h>

#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

// Static variables:
Mountain::Mountain() : Rille()
{
}

Mountain::Mountain(char* name, char* desc, int size, int angle,
                   int longitude, int latitude)
{
    Init(name, desc, size, angle, longitude, latitude);
}

void
Mountain::Init(char* name, char* desc, int size, int angle,
               int longitude, int latitude)
{
    Rille::Init(name, desc, size, angle, longitude, latitude);
}

Mountain::~Mountain()
{
}

void Mountain::PaintFeature(MoonCanvas* canvas, int cX, int cY, int radius,
                            int mirrored, int librationE, int librationN)
{
    canvas->SetColor(255, 100, 40);

    double longRadians = M_PI/180.* (GetLongitude() + librationE);
    double latRadians = M_PI/180. * (GetLatitude() + librationN);
    cX += mirrored * (int)(radius * sin(longRadians));
    cY += (int)(-radius * sin(latRadians));
    int dx, dy;

    // We don't have good angular or length data for most rilles.
    // So we use -1 as a code for rilles that haven't been checked yet,
    // and draw them as an X instead of a line.
    if (mAngle < 0)
    {
        if (mSize > 150)
            dx = dy = radius/20;
        else
            dx = dy = radius/40;
        canvas->DrawLine(cX - dx, cY - dy, cX + dx, cY + dy);
        canvas->DrawLine(cX - dx, cY + dy, cX + dx, cY - dy);
    }
    else
    {
        double length = mSize * radius / 1609.;
        dx = -(int)(length * cos(M_PI/180.*mAngle) * mirrored);
        dy = (int)(length * sin(M_PI/180.*mAngle));

        canvas->DrawLine(cX - dx, cY - dy - 1, cX + dx, cY + dy - 1);
        canvas->DrawLine(cX - dx, cY - dy + 1, cX + dx, cY + dy + 1);
    }
    //printf("Mountain from (%d, %d) to (%d, %d)\n",
    //       cX - dx, cY - dy, cX + dx, cY + dy);

    LabelName(canvas, cX - dx, cY - dy);
}

// Routines for looping over all known craters:
void
Mountain::InitMountains()
{
    if (!sMoonFeatureList)
        return;

    char* mountainFileName = DATADIR "montes.dat";
    FILE* cfile = fopen(mountainFileName, "r");
    if (!cfile)
        return;

    char line[BUFSIZ];
    while (fgets(line, sizeof line, cfile) != 0)
    {
        if (*line == '#')       // Skip comments
            continue;
        int latitude = (int)atof(line+30);
        if (line[36] == 'S')
            latitude = -latitude;
        int longitude = (int)atof(line+38);
        if (line[44] == 'W')
            longitude = -longitude;
        // Write a null after the name in the line buffer,
        // so we can pass it to strdup:
        int i;
        for (i=28; i>0 && line[i] == ' '; --i)
            ;
        if (i <= 0)
            return;
        line[i+1] = '\0';

        sMoonFeatureList[sNumFeatures++] = new Mountain(line, line+61,
                                                        (int)atof(line+47),
                                                        (int)atof(line+55),
                                                        longitude, latitude);
    }
    printf("Mountains loaded, %d features\n", sNumFeatures);
}


